import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import { GroupHeading, Item, MenuSection } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/navigation-next.js";
import Tooltip from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/tooltip.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import ConfirmationModal from "/apps/websight-admin/web-resources/ConfirmationModal.js";
import { setUrlHashValue } from "/apps/websight-admin/web-resources/services/SearchParamsService.js";
import BrowseScriptModal from "./BrowseScriptModal.js";
import ScriptService from "../services/ScriptService.js";
const ScriptsMenuContainer = styled.div`
    height: 100%;
    display: flex;
    flex-direction: column;

    & div:last-child {
        justify-content: flex-end;
        flex-grow: 1;
    }
`;
export default class ScriptsMenu extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      selectedScript: {}
    };
    this.openScript = this.openScript.bind(this);
    this.openNewScript = this.openNewScript.bind(this);
    this.onScriptSelect = this.onScriptSelect.bind(this);
    this.onScriptDelete = this.onScriptDelete.bind(this);
    this.onNewScriptClick = this.onNewScriptClick.bind(this);
  }

  openScript(script = this.state.selectedScript, getScriptCode = true) {
    this.props.onScriptChange(script, getScriptCode);
  }

  openNewScript() {
    setUrlHashValue('');
    this.openScript(ScriptService.emptyScript, false);
  }

  onScriptSelect(selectedScript) {
    setUrlHashValue(selectedScript.path);
    this.setState({
      selectedScript
    }, () => this.props.script.changed ? this.openScriptModal.open() : this.openScript());
  }

  onNewScriptClick() {
    this.props.script.changed ? this.newScriptModal.open() : this.openNewScript();
  }

  onScriptDelete() {
    this.props.onScriptDelete();
  }

  render() {
    const {
      script,
      recentScripts,
      onHelperClick
    } = this.props;
    return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(MenuSection, null, ({
      className
    }) => /*#__PURE__*/React.createElement(ScriptsMenuContainer, {
      className: `${className}`
    }, /*#__PURE__*/React.createElement(Item, {
      before: () => /*#__PURE__*/React.createElement("i", {
        className: "material-icons-outlined"
      }, "add"),
      text: "New script",
      onClick: this.onNewScriptClick
    }), /*#__PURE__*/React.createElement(Item, {
      before: () => /*#__PURE__*/React.createElement("i", {
        className: "material-icons-outlined"
      }, "folder"),
      text: "Browse scripts",
      onClick: () => this.browseScriptModal.open()
    }), /*#__PURE__*/React.createElement(GroupHeading, null, "Recent scripts"), recentScripts.map(recentScript => {
      return /*#__PURE__*/React.createElement(Tooltip, {
        key: recentScript.path,
        content: recentScript.path + ' (by ' + recentScript.lastModifiedBy + ')'
      }, /*#__PURE__*/React.createElement(Item, {
        before: () => /*#__PURE__*/React.createElement("i", {
          className: "material-icons"
        }, "insert_drive_file"),
        isActive: recentScript.path === script.path,
        key: recentScript.path,
        onClick: () => this.onScriptSelect(recentScript),
        text: recentScript.name,
        subText: recentScript.relativePath + ' (by ' + recentScript.lastModifiedBy + ')'
      }));
    }), /*#__PURE__*/React.createElement(Item, {
      before: () => /*#__PURE__*/React.createElement("i", {
        className: "material-icons-outlined"
      }, "help"),
      text: "Help",
      onClick: onHelperClick
    }))), /*#__PURE__*/React.createElement(BrowseScriptModal, {
      key: "browse-script-modal",
      ref: element => this.browseScriptModal = element,
      onScriptSelect: this.onScriptSelect,
      onScriptDelete: this.onScriptDelete
    }), /*#__PURE__*/React.createElement(ConfirmationModal, {
      key: "new-script-confirmation-modal",
      appearance: "warning",
      buttonText: "Open",
      heading: "Open new script",
      message: /*#__PURE__*/React.createElement("span", null, "You will lost all your unsaved changes. Are you sure you want to create a new script?"),
      onConfirm: this.openNewScript,
      ref: element => this.newScriptModal = element
    }), /*#__PURE__*/React.createElement(ConfirmationModal, {
      key: "open-script-confirmation-modal",
      appearance: "warning",
      buttonText: "Open",
      heading: "Open script",
      message: /*#__PURE__*/React.createElement("span", null, "You will lost all your unsaved changes. Are you sure you want to open ", this.state.selectedScript.name, "?"),
      onConfirm: this.openScript,
      ref: element => this.openScriptModal = element
    }));
  }

}