import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import Select, { CheckboxSelect } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/select.js";
import Tooltip from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/tooltip.js";
import { FilterOptionsContainer, FilterPatternContainer, FilterSortByContainer, HeaderFiltersContainer } from "/apps/websight-admin/web-resources/Containers.js";
import { debounce } from "/apps/websight-admin/web-resources/Utils.js";
import StatefulFilterInput from "/apps/websight-admin/web-resources/components/StatefulFilterInput.js";
const FILTER_OPTION_BUILT = 'built';
const FILTER_OPTION_NOT_BUILT = 'notBuilt';
const FILTER_OPTION_INSTALLED = 'installed';
const FILTER_OPTION_NOT_INSTALLED = 'notInstalled';
const filterOptions = [{
  label: 'Created by me',
  value: 'createdByMe'
}, {
  label: 'Built',
  value: FILTER_OPTION_BUILT
}, {
  label: 'Not built',
  value: FILTER_OPTION_NOT_BUILT
}, {
  label: 'Installed',
  value: FILTER_OPTION_INSTALLED
}, {
  label: 'Not installed',
  value: FILTER_OPTION_NOT_INSTALLED
}, {
  label: 'Modified',
  value: 'modified'
}, {
  label: 'Scheduled',
  value: 'scheduled'
}];

const getFilterOptionsByValues = values => {
  if (!values) {
    return [];
  }

  return filterOptions.filter(option => values.includes(option.value));
};

const extractValuesFromOptions = options => {
  options = options || [];
  return options.map(option => option.value);
};

const adjustFilterOptionsValuesOnChange = (newValues, previousValues) => {
  newValues = newValues || [];
  previousValues = previousValues || [];

  if (valueAdded(FILTER_OPTION_BUILT, newValues, previousValues)) {
    removeFromValues(FILTER_OPTION_NOT_BUILT, newValues);
  } else if (valueAdded(FILTER_OPTION_NOT_BUILT, newValues, previousValues)) {
    removeFromValues(FILTER_OPTION_BUILT, newValues);
  } else if (valueAdded(FILTER_OPTION_INSTALLED, newValues, previousValues)) {
    removeFromValues(FILTER_OPTION_NOT_INSTALLED, newValues);
  } else if (valueAdded(FILTER_OPTION_NOT_INSTALLED, newValues, previousValues)) {
    removeFromValues(FILTER_OPTION_INSTALLED, newValues);
  }

  return newValues;
};

const valueAdded = (optionValue, newValues, previousValues) => {
  return newValues.includes(optionValue) && !previousValues.includes(optionValue);
};

const removeFromValues = (value, values) => {
  const indexToRemove = values.indexOf(value);

  if (indexToRemove >= 0) {
    values.splice(indexToRemove, 1);
  }
};

const defaultSortBySelectOption = {
  label: 'Last used',
  value: 'lastUsed'
};
const sortBySelectOptions = [defaultSortBySelectOption, {
  label: 'Name',
  value: 'name'
}, {
  label: 'Last modified',
  value: 'lastModified'
}, {
  label: 'Installation date',
  value: 'installationDate'
}, {
  label: 'Recently added',
  value: 'recentlyAdded'
}];

const getSortByOptionByValue = value => {
  if (!value) {
    return defaultSortBySelectOption;
  }

  return sortBySelectOptions.find(option => option.value === value);
};

export default class HeaderBottomBar extends React.Component {
  render() {
    const {
      params,
      limitExceeded
    } = this.props;
    return /*#__PURE__*/React.createElement(HeaderFiltersContainer, null, /*#__PURE__*/React.createElement(FilterPatternContainer, null, /*#__PURE__*/React.createElement(StatefulFilterInput, {
      onClear: () => {
        this.props.findPackages({
          filter: '',
          pageNumber: 1
        });
      },
      onChange: event => {
        const targetValue = event.target.value;
        this.debounceTimerId = debounce(() => this.props.findPackages({
          filter: targetValue,
          pageNumber: 1
        }), this.debounceTimerId);
      },
      value: params.filter
    })), /*#__PURE__*/React.createElement(FilterOptionsContainer, null, /*#__PURE__*/React.createElement(CheckboxSelect, {
      placeholder: "Choose a filter",
      spacing: "compact",
      options: filterOptions,
      value: getFilterOptionsByValues(params.filterOptions),
      onChange: selectedOptions => {
        const adjustedValues = adjustFilterOptionsValuesOnChange(extractValuesFromOptions(selectedOptions), params.filterOptions);
        this.props.findPackages({
          filterOptions: adjustedValues,
          pageNumber: 1
        });
      }
    })), /*#__PURE__*/React.createElement(FilterSortByContainer, null, limitExceeded ? /*#__PURE__*/React.createElement(Tooltip, {
      content: "Sorting disabled due to high number of packages",
      delay: 0
    }, /*#__PURE__*/React.createElement(Select, {
      placeholder: "Sort By",
      spacing: "compact",
      isDisabled: true
    })) : /*#__PURE__*/React.createElement(Select, {
      placeholder: "Sort By",
      spacing: "compact",
      options: sortBySelectOptions,
      defaultValue: defaultSortBySelectOption,
      value: getSortByOptionByValue(params.sortBy),
      onChange: selectedOption => {
        this.props.findPackages({
          sortBy: selectedOption.value,
          pageNumber: 1
        });
      }
    })));
  }

}