import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import { Checkbox } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/checkbox.js";
import { Fieldset } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/form.js";
import { CreatableSelect } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/select.js";
import TextArea from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/textarea.js";
import Textfield from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/textfield.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import FilePicker from "/apps/websight-admin/web-resources/FilePicker.js";
import { getOptionByValue } from "../../utils/CommonUtils.js";
import PackageThumbnail from "../PackageThumbnail.js";
const GeneralTabContainer = styled.div`
    width: 100%;
`;

const getGroupsAsOptions = groups => {
  return groups.filter(group => group.name !== ':no_group').map(group => ({
    label: group.name,
    value: group.name
  }));
};

export default class PackageModalGeneralTab {
  constructor(props) {
    this.props = props;
  }

  render() {
    const {
      packageData,
      defaultGroup,
      showThumbnail,
      showDeleteThumbnailCheckbox
    } = this.props;
    const groupsOptions = getGroupsAsOptions(this.props.groups);
    const actualDefaultGroup = defaultGroup || packageData.group || 'my_packages';
    return /*#__PURE__*/React.createElement(GeneralTabContainer, null, /*#__PURE__*/React.createElement(Textfield, {
      autocomplete: "off",
      defaultValue: packageData.name,
      isRequired: true,
      name: "name",
      label: "Name",
      autoFocus: true
    }), /*#__PURE__*/React.createElement(CreatableSelect, {
      isClearable: true,
      name: "group",
      label: "Group",
      options: groupsOptions,
      defaultValue: getOptionByValue(actualDefaultGroup, groupsOptions)
    }), /*#__PURE__*/React.createElement(Textfield, {
      autocomplete: "off",
      defaultValue: packageData.version,
      name: "version",
      label: "Version"
    }), /*#__PURE__*/React.createElement(TextArea, {
      autocomplete: "off",
      defaultValue: packageData.description,
      name: "description",
      label: "Description",
      resize: "vertical"
    }), /*#__PURE__*/React.createElement(Fieldset, {
      legend: "Thumbnail"
    }, showThumbnail && packageData.thumbnail && /*#__PURE__*/React.createElement("div", {
      style: {
        display: 'flex',
        marginBottom: '16px'
      },
      onMouseEnter: () => this.props.onShowDeleteThumbnailChange(true),
      onMouseLeave: () => this.props.onShowDeleteThumbnailChange(false)
    }, /*#__PURE__*/React.createElement(PackageThumbnail, {
      thumbnailData: packageData.thumbnail,
      style: {
        marginTop: '8px'
      },
      size: 64
    }), (showDeleteThumbnailCheckbox || this.props.deleteThumbnail) && /*#__PURE__*/React.createElement("div", {
      style: {
        display: 'flex',
        alignItems: 'center'
      }
    }, /*#__PURE__*/React.createElement(Checkbox, {
      hideLabel: true,
      label: "Delete thumbnail",
      name: "deleteThumbnail",
      value: "true",
      onChange: () => {
        // TODO: Correct this workaround when WS-376 is fixed
        this.props.onDeleteThumbnailChange('thumbnail');
      }
    }))), !this.props.deleteThumbnail && /*#__PURE__*/React.createElement("div", {
      style: {
        marginTop: '-6px'
      }
    }, /*#__PURE__*/React.createElement(FilePicker, {
      name: "thumbnail",
      accept: "image/png"
    }))));
  }

}