function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import DropdownMenu, { DropdownItem, DropdownItemGroup } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/dropdown-menu.js";
import Tooltip from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/tooltip.js";
import styled from "/apps/websight-atlaskit-esm/web-resources/styled-components.js";
import { fetchApplicableActions } from "/apps/websight-admin/web-resources/utils/ExtraActionsUtil.js";
import ChangelogWriteService from "../../services/ChangelogWriteService.js";
const ResourceOperationsMenuContainer = styled.div`
    background: #f4f5f7;
    position: absolute;
    top: 0;
    z-index: 200;
    right: 20px;
`;
const ResourceOperationsContextMenuContainer = styled.div`
    background: #f4f5f7;
    position: absolute;
    top: 0;
    z-index: 300;
    left: 30px;
`;
const OperationIcon = styled.i`
    font-size: 14px;
    vertical-align: middle;
    position: relative;
    bottom: 2px;
    padding-right: 5px;
`;
export default class ResourceOperationsMenu extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      isOpen: this.props.selected || false,
      extraActions: this.props.extraActions
    };
    this.close = this.close.bind(this);
    this.open = this.open.bind(this);
  }

  componentDidUpdate(prevProps) {
    if (prevProps.selected !== this.props.selected) {
      this.setState({
        isOpen: this.props.selected
      });
    }
  }

  close() {
    if (!this.props.selected) {
      this.setState({
        isOpen: false
      });
    }
  }

  open() {
    if (!this.props.selected) {
      this.setState({
        isOpen: true
      });
    }
  }

  filterActions(event) {
    if (event.isOpen) {
      fetchApplicableActions(this.props.extraActions, this.props.resource.path, actions => {
        this.setState({
          extraActions: actions
        });
      });
    }
  }

  render() {
    const {
      changelog,
      children,
      onChangelogUpdate,
      onCreateResource,
      onRenameResource,
      resource,
      onCopyMoveResource,
      onTreeRefresh,
      onNodeRefresh,
      loadingResources
    } = this.props;
    const {
      extraActions
    } = this.state;
    const createResourceOption = {
      label: 'Create Resource',
      icon: 'add',
      props: {
        onClick: () => onCreateResource(resource)
      }
    };
    const options = [createResourceOption, {
      label: 'Delete Resource',
      icon: 'delete',
      props: {
        onClick: () => {
          ChangelogWriteService.removeResource(changelog, resource);
          onChangelogUpdate(changelog);
        }
      }
    }, {
      label: 'Refresh Node',
      icon: 'refresh',
      props: {
        onClick: onNodeRefresh,
        isLoading: loadingResources
      }
    }, {
      label: 'Rename Resource',
      icon: 'edit',
      props: {
        onClick: onRenameResource
      }
    }, {
      label: 'Move Resource',
      icon: 'swap_vert',
      props: {
        onClick: () => onCopyMoveResource(resource, true)
      }
    }, {
      label: 'Copy Resource',
      icon: 'content_copy',
      props: {
        onClick: () => onCopyMoveResource(resource, false)
      }
    }];
    const rootOptions = [createResourceOption, {
      label: 'Refresh Tree',
      icon: 'refresh',
      props: {
        onClick: onTreeRefresh,
        isLoading: loadingResources
      }
    }];
    return /*#__PURE__*/React.createElement("div", {
      style: {
        position: 'relative'
      }
    }, children, this.state.isOpen && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(ResourceOperationsContextMenuContainer, null, /*#__PURE__*/React.createElement(DropdownMenu, {
      defaultOpen: resource.showContextMenu,
      onOpenChange: e => {
        if (!e.isOpen) {
          resource.showContextMenu = false;
        }
      }
    }, /*#__PURE__*/React.createElement(DropdownItemGroup, null, (resource.path === '/' ? rootOptions : options).map(({
      icon,
      label,
      props
    }) => /*#__PURE__*/React.createElement(DropdownItem, _extends({
      key: label
    }, props, {
      spacing: "compact"
    }), /*#__PURE__*/React.createElement(OperationIcon, {
      className: "material-icons"
    }, icon), label)), extraActions && extraActions.map((action, index) => {
      return /*#__PURE__*/React.createElement(DropdownItem, {
        spacing: "compact",
        key: index,
        onClick: () => action.onClick(resource)
      }, /*#__PURE__*/React.createElement(OperationIcon, {
        className: "material-icons"
      }, action.icon), action.label);
    })))), /*#__PURE__*/React.createElement(ResourceOperationsMenuContainer, null, /*#__PURE__*/React.createElement(DropdownMenu, {
      defaultOpen: false,
      triggerType: 'button',
      onOpenChange: event => this.filterActions(event),
      triggerButtonProps: {
        iconBefore: /*#__PURE__*/React.createElement(OperationIcon, {
          className: "material-icons"
        }, "more_vert")
      }
    }, /*#__PURE__*/React.createElement(DropdownItemGroup, null, (resource.path === '/' ? rootOptions : options).map(({
      icon,
      label,
      props
    }) => /*#__PURE__*/React.createElement(Tooltip, {
      key: label,
      content: label,
      delay: 0
    }, /*#__PURE__*/React.createElement(DropdownItem, _extends({}, props, {
      spacing: "compact"
    }), /*#__PURE__*/React.createElement(OperationIcon, {
      className: "material-icons"
    }, icon)))), extraActions && extraActions.map((action, index) => {
      return /*#__PURE__*/React.createElement(Tooltip, {
        key: action.label,
        content: action.label,
        delay: 0
      }, /*#__PURE__*/React.createElement(DropdownItem, {
        spacing: "compact",
        key: index,
        onClick: () => action.onClick(resource)
      }, /*#__PURE__*/React.createElement(OperationIcon, {
        className: "material-icons"
      }, action.icon)));
    }))))));
  }

}