import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import Button, { ButtonGroup } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/button.js";
import ModalDialog, { ModalTransition } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/modal-dialog.js";
import TextArea from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/textarea.js";
import TextField from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/textfield.js";
import Form, { FormFooter } from "/apps/websight-rest-atlaskit-client/web-resources/js/Form.js";
import AuthorizableAutosuggestion from "/apps/websight-autosuggestion-esm/web-resources/AuthorizableAutosuggestion.js";
import GroupAutosuggestion from "/apps/websight-autosuggestion-esm/web-resources/GroupAutosuggestion.js";
import GroupService from "./../services/GroupService.js";
import { PRINCIPAL_EVERYONE } from "../utils/UserManagerConstants.js";
import { filterOutEveryonePrincipal } from "../utils/AuthorizableUtil.js";
export default class GroupFormModal extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      isOpen: false,
      group: { ...GroupService.emptyGroupData
      }
    };
    this.open = this.open.bind(this);
    this.close = this.close.bind(this);
    this.onSubmitSuccess = this.onSubmitSuccess.bind(this);
    this.onFormSubmit = this.onFormSubmit.bind(this);
  }

  open() {
    this.setState({
      isOpen: true,
      group: { ...GroupService.emptyGroupData,
        ...(!this.props.create ? this.props.group : {})
      }
    });
  }

  close() {
    this.setState({
      isOpen: false
    });
  }

  onSubmitSuccess(data) {
    this.close();
    this.props.onSaveSuccess && this.props.onSaveSuccess(data);
  }

  onFormSubmit(...args) {
    this.setState({
      group: args[0]
    });

    if (this.props.create) {
      GroupService.createGroup(...args);
    } else {
      GroupService.updateGroup(...args);
    }
  }

  render() {
    const {
      isOpen,
      group
    } = this.state;
    const form = /*#__PURE__*/React.createElement(Form, {
      onSubmit: this.onFormSubmit,
      onSuccess: this.onSubmitSuccess
    }, ({
      submitted
    }) => /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(TextField, {
      autocomplete: "off",
      label: "ID",
      name: "id",
      isRequired: true,
      isDisabled: !this.props.create,
      defaultValue: group.id
    }), /*#__PURE__*/React.createElement(TextField, {
      autocomplete: "off",
      label: "Display name",
      name: "displayName",
      defaultValue: group.displayName
    }), /*#__PURE__*/React.createElement(TextArea, {
      autocomplete: "off",
      label: "Description",
      name: "description",
      defaultValue: group.description
    }), group.id !== PRINCIPAL_EVERYONE && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(GroupAutosuggestion, {
      defaultValue: [...(filterOutEveryonePrincipal(group.parentGroups) || [])],
      isMulti: true,
      label: "Parent groups",
      name: "parentGroups",
      filterOut: [PRINCIPAL_EVERYONE]
    }), /*#__PURE__*/React.createElement(AuthorizableAutosuggestion, {
      defaultValue: [...(filterOutEveryonePrincipal(group.members) || [])],
      isMulti: true,
      label: "Members",
      name: "members",
      noOptionsMessage: inputValue => `No members found for "${inputValue}"`,
      noOptionEmptyMessage: "Start typing to find a member",
      filterOut: [PRINCIPAL_EVERYONE]
    })), /*#__PURE__*/React.createElement(FormFooter, null, /*#__PURE__*/React.createElement(ButtonGroup, null, /*#__PURE__*/React.createElement(Button, {
      appearance: "primary",
      type: "submit",
      isLoading: submitted
    }, this.props.create ? 'Create' : 'Save'), /*#__PURE__*/React.createElement(Button, {
      appearance: "subtle",
      onClick: this.close,
      isDisabled: submitted
    }, "Cancel")))));
    return /*#__PURE__*/React.createElement(ModalTransition, null, isOpen && /*#__PURE__*/React.createElement(ModalDialog, {
      heading: this.props.create ? 'Create new group' : 'Edit group',
      onClose: this.close
    }, form));
  }

}