import React from "/apps/websight-atlaskit-esm/web-resources/react.js";
import { Cell, Row } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/table-tree.js";
import DropdownMenu, { DropdownItem } from "/apps/websight-atlaskit-esm/web-resources/@atlaskit/dropdown-menu.js";
import { TableRowActionButtonsContainer } from "/apps/websight-admin/web-resources/Containers.js";
import ConfirmationModal from "/apps/websight-admin/web-resources/ConfirmationModal.js";
import SystemUserService from "../services/SystemUserService.js";
import AuthorizableName from "./AuthorizableName.js";
import { fetchApplicableActions } from "/apps/websight-admin/web-resources/utils/ExtraActionsUtil.js";
export default class SystemUserRow extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      extraActions: props.extraActions
    };
  }

  deleteUser() {
    const {
      user
    } = this.props;
    SystemUserService.deleteUser(user.id, () => {
      if (this.props.onAuthorizableAction) {
        this.props.onAuthorizableAction('delete', user);
      }

      this.props.onUpdate();
    }, () => this.deleteConfirmationModal.close());
  }

  actionButtons() {
    const {
      extraActions
    } = this.state;
    return /*#__PURE__*/React.createElement(TableRowActionButtonsContainer, null, /*#__PURE__*/React.createElement(DropdownMenu, {
      trigger: "",
      triggerType: "button",
      onOpenChange: event => {
        if (event.isOpen) {
          fetchApplicableActions(this.props.extraActions, this.props.user.path, actions => {
            this.setState({
              extraActions: actions
            });
          });
        }
      }
    }, extraActions && extraActions.map((action, index) => {
      return /*#__PURE__*/React.createElement(DropdownItem, {
        key: index,
        onClick: () => action.onClick(this.props.user.path)
      }, action.label);
    }), /*#__PURE__*/React.createElement(DropdownItem, {
      onClick: () => this.deleteConfirmationModal.open()
    }, "Delete")), extraActions && extraActions.map(action => {
      return action.modal(this.props.user);
    }), /*#__PURE__*/React.createElement(ConfirmationModal, {
      buttonText: 'Delete',
      heading: 'Delete user',
      appearance: "danger",
      message: /*#__PURE__*/React.createElement("span", null, "Are you sure you want to permanently delete ", /*#__PURE__*/React.createElement("b", null, this.props.user.id), "?"),
      onConfirm: () => this.deleteUser(),
      ref: element => this.deleteConfirmationModal = element
    }));
  }

  render() {
    const {
      user
    } = this.props;

    const userRowCells = () => [/*#__PURE__*/React.createElement(Cell, {
      key: `${user.id}-id`
    }, /*#__PURE__*/React.createElement(AuthorizableName, {
      authorizable: user,
      icon: "settings"
    })), /*#__PURE__*/React.createElement(Cell, {
      singleLine: true,
      key: `${user.id}-actions`
    }, this.actionButtons())];

    return /*#__PURE__*/React.createElement(Row, {
      key: user.id,
      itemId: user.id
    }, userRowCells());
  }

}